// Copyright 2018 The Prometheus Authors
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Scrape `information_schema.INNODB_CMPMEM`.

package collector

import (
	"context"
	"database/sql"

	"github.com/go-kit/log"
	"github.com/prometheus/client_golang/prometheus"
)

const recordsCapacityQuery = `
			select (used_pages/total_pages)+0.05 'used' 
			from ndbinfo.memoryusage 
			where memory_type='Data memory' limit 1
            `

// Metric descriptors.
var (
	infoRecordsCapacity = prometheus.NewDesc(
		prometheus.BuildFQName(namespace, informationSchema, "records_capacity_used"),
		"Show counter records capacity used for allocated memory ",
		nil,nil,
	)
)

// ScrapeInnodbCmp collects from `information_schema.innodb_cmp`.
type ScrapeRecordsCapacity struct{}

// Name of the Scraper. Should be unique.
func (ScrapeRecordsCapacity) Name() string {
	return "ndbinfo.recordsused"
}

// Help describes the role of the Scraper.
func (ScrapeRecordsCapacity) Help() string {
	return "Collect metrics from ndbinfo.memoryusage"
}

// Version of MySQL from which scraper is available.
func (ScrapeRecordsCapacity) Version() float64 {
	return 5.5
}

// Scrape collects data from database connection and sends it over channel as prometheus metric.
func (ScrapeRecordsCapacity) Scrape(ctx context.Context, db *sql.DB, ch chan<- prometheus.Metric, logger log.Logger) error {
	infoRecordsCapacityRows, err := db.QueryContext(ctx, recordsCapacityQuery)
	if err != nil {
		return err
	}
	defer infoRecordsCapacityRows.Close()

	var (
		records_used  float64
	)

	for infoRecordsCapacityRows.Next() {
		if err := infoRecordsCapacityRows.Scan(
			&records_used,
		); err != nil {
			return err
		}

		ch <- prometheus.MustNewConstMetric(infoRecordsCapacity, prometheus.GaugeValue, records_used)
	}
	return nil
}

// check interface
var _ Scraper = ScrapeRecordsCapacity{}
