// Copyright 2018 The Prometheus Authors
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Scrape `information_schema.INNODB_CMPMEM`.

package collector

import (
	"context"
	"database/sql"

	"github.com/go-kit/log"
	"github.com/prometheus/client_golang/prometheus"
)

const slowQueryQuery = `
			SELECT
			count(stmts.DIGEST_TEXT) sq
			FROM (performance_schema.events_statements_summary_by_digest stmts
			JOIN sys.x$ps_digest_95th_percentile_by_avg_us top_percentile ON((ROUND((stmts.AVG_TIMER_WAIT / 1000000),0) >= top_percentile.avg_us)))
			WHERE last_seen >= NOW() - INTERVAL 1 DAY
			AND ROUND((stmts.AVG_TIMER_WAIT / 1000000),0)>10000000
			ORDER BY stmts.AVG_TIMER_WAIT DESC;
                `

// Metric descriptors.
var (
	infoSlowQuery = prometheus.NewDesc(
		prometheus.BuildFQName(namespace, informationSchema, "slow_query24h"),
		"Show counter slow query last 24 hours",
		nil,nil,
	)
)

// ScrapeInnodbCmp collects from `information_schema.innodb_cmp`.
type ScrapeSlowQuery struct{}

// Name of the Scraper. Should be unique.
func (ScrapeSlowQuery) Name() string {
	return "performance_schema.events_statements_summary_by_digest"
}

// Help describes the role of the Scraper.
func (ScrapeSlowQuery) Help() string {
	return "Collect metrics from performance_schema.events_statements_summary_by_digest"
}

// Version of MySQL from which scraper is available.
func (ScrapeSlowQuery) Version() float64 {
	return 5.5
}

// Scrape collects data from database connection and sends it over channel as prometheus metric.
func (ScrapeSlowQuery) Scrape(ctx context.Context, db *sql.DB, ch chan<- prometheus.Metric, logger log.Logger) error {
	informationSchemaInnodbCmpMemRows, err := db.QueryContext(ctx, slowQueryQuery)
	if err != nil {
		return err
	}
	defer informationSchemaInnodbCmpMemRows.Close()

	var (
		count_slow_query  float64
	)

	for informationSchemaInnodbCmpMemRows.Next() {
		if err := informationSchemaInnodbCmpMemRows.Scan(
			&count_slow_query,
		); err != nil {
			return err
		}

		ch <- prometheus.MustNewConstMetric(infoSlowQuery, prometheus.GaugeValue, count_slow_query)
	}
	return nil
}

// check interface
var _ Scraper = ScrapeSlowQuery{}
